/*************************************************************
*
*  模块名 ：data_acq
*  日期	：2021年3月4日
*  说明	：数据采集传输系统adc驱动模块
*
****************************************************************/


module adc_driver(
	Clk,
	Rst_n,
	START,
	DIN_R_IN,
	DOUT,
	
	cs,
	sclk_o,
	dac_data,
	DIN,
	RE_DONE,
	ADC_STATE,
	ADC_START
);

	input Clk;
	input Rst_n;
	input START;
	input [2:0]DIN_R_IN; //采样管道参数输入
	input DOUT;  //ADC芯片输出的数据,fpga接收
	input ADC_START;
	
	output reg cs;  //输出片选时钟
	output reg sclk_o;  //输出sclk时钟
	output reg [11:0]dac_data;  //缓存dout的数据
	output reg DIN;  //ADC通道参数设置
	output RE_DONE;  //采样完成标志
	output ADC_STATE;  //adc状态
	
	parameter clk_cnt_p = 8'd12;  //0、1、2 6.25M
	
	reg [7:0]clk_cnt;  //分频计数
	reg sclk;  //adc时钟
	reg en;  //使能信号
	reg [7:0]sclk_cnt;

	reg [11:0]dac_data_r;  //缓存接收的数据，接收满后输出给dac_data
	reg [2:0]DIN_R;
	
//START置1，使能信号
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		en <= 1'b0;
	else if(START && ADC_START)  //控制采集100次后停止采集
		en <= 1'b1;
	else if(RE_DONE)
		en <= 1'b0;
	else 
		en <= en;
//通道值设置
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		DIN_R <= 3'd0;
	else if(en)
		DIN_R <= DIN_R_IN;
	else 
		DIN_R <= DIN_R;
		
//时钟
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		clk_cnt <= 8'd0;
	else if(clk_cnt == clk_cnt_p)
		clk_cnt <= 8'd0;
	else if(en)
		clk_cnt <= clk_cnt + 1'b1;
	else 
		clk_cnt <= clk_cnt;
	//产生sclk时钟
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)	
		sclk <= 1'b0;
	else if(en && (clk_cnt == clk_cnt_p))
		sclk <= 1'b1;
	else 
		sclk <= 1'b0;
		
	//sclk计数
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		sclk_cnt <= 8'd0;
	else if((sclk_cnt == 8'd33) && sclk)
		sclk_cnt <= 8'd0;
	else if(sclk && en)  //判断条件加上sclk和en确保只有使能时自加
		sclk_cnt <= sclk_cnt + 1'b1;
	else 
		sclk_cnt <= sclk_cnt;
		
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)begin
		DIN <= 1'd0;
		dac_data_r <= 12'd0;
		sclk_o <= 1'b1;  //初始状态为1
		cs <= 1'b1;  //初始状态为1
	end
	else if(en && sclk)begin  //加入sclk，让adc的操作在sclk的边沿执行，而不是clk的边沿
		case(sclk_cnt)
			0: begin cs <= 1'b0;sclk_o <= 1'b1;end
			1: begin sclk_o <= 1'b0;DIN <= 1'b0; end  //上升沿到来后读取数据
			2: begin sclk_o <= 1'b1;end
			3: begin sclk_o <= 1'b0;end
			4: begin sclk_o <= 1'b1;end
			5: begin sclk_o <= 1'b0;DIN <= DIN_R[2];end  //sclk上升沿采集DIN数据(先给数据再拉高，确保上升沿到来时数据已经变化了)
			6: begin sclk_o <= 1'b1;end
			7: begin sclk_o <= 1'b0;DIN <= DIN_R[1];end
			8: begin sclk_o <= 1'b1;end
			9: begin sclk_o <= 1'b0;DIN <= DIN_R[0];end
			10: begin sclk_o <= 1'b1;dac_data_r[11] <= DOUT;end  //sclk下降沿输出DOUT数据
			11: begin sclk_o <= 1'b0;end  
			12: begin sclk_o <= 1'b1;dac_data_r[10] <= DOUT;end
			13: begin sclk_o <= 1'b0;end
			14: begin sclk_o <= 1'b1;dac_data_r[9] <= DOUT;end
			15: begin sclk_o <= 1'b0;end
			16: begin sclk_o <= 1'b1;dac_data_r[8] <= DOUT; end
			17: begin sclk_o <= 1'b0; end
			18: begin sclk_o <= 1'b1;dac_data_r[7] <= DOUT; end
			19: begin sclk_o <= 1'b0; end
			20: begin sclk_o <= 1'b1;dac_data_r[6] <= DOUT; end
			21: begin sclk_o <= 1'b0; end
			22: begin sclk_o <= 1'b1;dac_data_r[5] <= DOUT; end
			23: begin sclk_o <= 1'b0; end
			24: begin sclk_o <= 1'b1;dac_data_r[4] <= DOUT; end
			25: begin sclk_o <= 1'b0; end
			26: begin sclk_o <= 1'b1;dac_data_r[3] <= DOUT; end
			27: begin sclk_o <= 1'b0; end
			28: begin sclk_o <= 1'b1;dac_data_r[2] <= DOUT; end
		   29: begin sclk_o <= 1'b0; end
			30: begin sclk_o <= 1'b1;dac_data_r[1] <= DOUT; end
			31: begin sclk_o <= 1'b0; end
			32: begin sclk_o <= 1'b1;dac_data_r[0] <= DOUT; end
			33: begin cs <= 1'b1;end
			default: begin cs = 1'b1;end
		endcase
	end
		
	assign RE_DONE = ((sclk_cnt == 8'd33)&&(sclk)&&en);  //计数33次一次采样完成

//接收完毕给数据输出	
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		dac_data <= 1'b0;
	else if(RE_DONE)
		dac_data <= dac_data_r;
	else	
		dac_data <= dac_data;
	
	
	assign ADC_STATE = cs;
endmodule 
	
	

	