/*************************************************************
*
*  模块名 ：data_acq
*  日期	：2021年3月4日
*  说明	：数据采集传输系统控制模块
*
****************************************************************/

module ctrl(
		Clk,
		Rst_n,
		wrreq,
		rdreq,
		Key_flag,
		Key_state,
		
		send_en,
		ADC_START,
		//DIN_R_IN,
		RE_DONE,
		ADC_STATE,
		full,
		fifo_data,
		TX_DONE,
		tx_data,
		empty,
		usedw,
		START,
		acq_num,
		switch
	);
	
	reg [7:0]acq_num_count;
	
	
	input [7:0]acq_num;
	input Clk;
	input Rst_n;
	
	input Key_state;  //按键状态
	input Key_flag;  //按键按下标志位
	input RE_DONE;
	input ADC_STATE;
	input [11:0]fifo_data;
	input TX_DONE;
	input [7:0]switch;
	
	output reg ADC_START;
	//output [2:0]DIN_R_IN;
	output reg wrreq;
	output reg rdreq;
	output reg send_en;
	output reg [7:0]tx_data;
	output reg START;
	
	input full;
	input empty;
	input [7:0]usedw;
	reg [1:0]tx_cnt;	
	reg [7:0]RE_DONE_CNT;
	reg [3:0]state;
	reg S1_flag;
	reg [7:0]tx_cnt_r;
	reg S2_flag;
//	reg wrreq_flag;
	reg tx_flag;
//	reg [7:0]switch_r;
//	reg switch_flag;
//adc采样管脚选择
	//assign DIN_R_IN = 3'd5;
	
//	always@(posedge Clk or negedge Rst_n)
//	if(!Rst_n)begin
//		switch_flag <= 1'd1;
//		switch_r <= 8'd0;
//	end
//	else if(switch_flag && switch == 8'h11)begin
//		switch_flag <= 1'd0;
//		switch_r <= 8'd1;
//	end
//	else 
//		switch_r <= 8'd0;
	
	
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		acq_num_count <= 8'd100;
	else if(START)
		acq_num_count = acq_num;
	
//按键按下开始采集,再按一次停止采集
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)begin
		START <= 1'd0;
		ADC_START <= 1'b0;
	end
	else if((Key_flag && !Key_state) || switch == 8'h11)begin  //按键按下
		ADC_START <= 1'b1;
		START <= 1'b1;
		//send_en <= 1'b1;
	end
	else if(RE_DONE_CNT == acq_num_count)begin
		ADC_START <= 1'd0;  //采集满100次停止采集
		START <= 1'b0;
	end 
	else begin
		START <= START;
		ADC_START <= ADC_START;
	end
	
	
	//计数采集100次
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		RE_DONE_CNT <= 8'd0;
	else if(RE_DONE)
		RE_DONE_CNT <= RE_DONE_CNT + 1'b1;
	else if(RE_DONE_CNT == acq_num_count)begin  //采集100次置0
		RE_DONE_CNT <= 8'd0;
		//ADC_START <= 1'd0;  //采集满100次停止采集
	end
	else 
		RE_DONE_CNT <= RE_DONE_CNT;
	
	
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)begin
		wrreq <= 1'b0;
		//wrreq_flag <= 1'b1;
	end
	else if(RE_DONE)begin //转换完成就读
		wrreq <= 1'b1;
		//wrreq_flag <= 1'b0;
	end
	else if(usedw == acq_num_count)
		wrreq <= 1'b0;
//	else if(tx_cnt_r == acq_num_count)
//		wrreq_flag <= 1'b1; //100次读取和发送完成，使能写数据
	else
		wrreq <= 1'b0;
		
//	always@(posedge Clk or negedge Rst_n)
//	if(!Rst_n)
//		rdreq <= 1'b0;
//	else if(!empty)begin  //存数据了就读取
//		rdreq <= 1'b1;
//		send_en <= 1'b1;
//	end
//	else if(tx_cnt == 2'd2)begin
//		send_en <= 1'b0;
//	end
//	else 
//		rdreq <= 1'b0;
		
	//发送计数
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		tx_cnt <= 2'd0;
	else if(TX_DONE)
		tx_cnt <= tx_cnt + 1'b1;
	else if(tx_cnt == 2'd2)
		tx_cnt <= 2'd0;
	else 
		tx_cnt <= tx_cnt;
		
	//将12位数据分两次发送
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		tx_data <= 8'd0;
	else begin
		case(tx_cnt)
			0: tx_data <= fifo_data[7:0];
			1: tx_data <= {4'b0000,fifo_data[11:8]};
		endcase
	end
	
	
//	//数据发送状态机
	localparam
		S0 = 4'b0001,  //等待采集完成
		S1 = 4'b0010,  //发送低八位
		S2 = 4'b0100,  //等待第一次发送完成，并发送高八位
		S3 = 4'b1000;  //等待第二次发送完成，返回等待采集
		
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)begin
		state <= S0;
		rdreq <= 1'd0;
		send_en <= 1'd0;
		S1_flag <= 1'd0;
		S2_flag <= 1'd0;
		tx_cnt_r <= 8'd0;
		tx_flag <= 1'd0;
	end
	else begin
		case(state)
			S0:  //等待采集完成
				if(usedw == acq_num_count)begin    //采集满100个
					state <= S1;
					rdreq <= 1'b1;
					S1_flag <= 1'b1;
					tx_flag <= 1'b1;
				end
				else if((tx_cnt_r >= 8'd1)&&(tx_cnt_r <= (acq_num_count -1))&&(tx_flag))begin
					rdreq <= 1'b1;
					state <= S1;
					S1_flag <= 1'b1;
				end
				else
					state <= S0;
					
			S1:  //发送第一次数据(高8位)
				if((!empty)&&(S1_flag))begin  //fifo有数据时，马上停止读取（只读1位） 只进一次
					rdreq <= 1'b0;
					send_en <= 1'b1;  //读取后马上开始发送
					S1_flag <= 1'b0;
				end
				else if(TX_DONE)begin  //
					state <= S2;  //当TX_DONE到来以后，低4位发送完成。进入S2
					S2_flag <= 1'd1;
					end
				else 
					state <= S1;  //其余情况回S0等待数据满100开始
					
			S2:
				if(S2_flag)begin
					send_en <= 1'd1;
					S2_flag <= 1'd0;
				end
				else if(TX_DONE)begin
					tx_cnt_r <= tx_cnt_r + 1'b1;
					send_en <= 1'd0;
					state <= S0;
				end
				else if(tx_cnt_r == (acq_num_count - 1))begin
					tx_flag <= 1'b0;
					tx_cnt_r = 8'd0;
					send_en <= 1'd0;
					state <= S0;
				end
				else
					state <= S2; //第二次发送完成回s0等待
				
			endcase		
	end
	
endmodule 
