/*************************************************************
*
*  模块名 ：data_acq
*  日期	：2021年3月4日
*  说明	：数据采集传输系统顶层测试文件
*	指令	：555A_采样开关_采样位数_采样通道_fo（支持串口触发采样和按键S1触发采样，默认采样数量100个，采样通道00）
			示例：555A006405F0  --> 采样开关关，采样数量100个，采样通道05
				  555A116405F0  --> 采样开关开，采样数量100个，采样通道05
****************************************************************/

module data_acq(
	Clk,
	Rst_n,
	DOUT,
	
	cs,
	sclk_o,
	Key_in,
	Rs232_Tx,
	DIN,  //选择的采样管道输出
	
	dac_cs_n,
	dac_din,
	dac_sclk,
	Rs232_Rx,
	led
);	
	input Clk;
	input Rst_n;
	input DOUT;  //adc芯片输出的数据
	input Key_in;  //按键输入
	input Rs232_Rx;
	
	output cs;
	output sclk_o;
	output DIN;
	output Rs232_Tx;
	output led;
	
	wire Tx_Done;
	wire uart_state;
	wire ADC_START;  //使能adc采样
	wire [2:0]DIN_R_IN;  //adc采样管脚选择
	wire [11:0]adc_data;  //adc_driver采集后输出的数据
	wire RE_DONE;  //adc采样转换完成
	wire ADC_STATE;  //adc采样的状态
	
	wire wrreq;  //fifo写使能
	wire rdreq;  //fifo读使能
	wire send_en;  //串口写使能
	wire Key_flag;  //按键按下标志
	wire Key_state;  //按键状态

	wire almost_empty;
	wire almost_full;
	wire empty;
	wire sclr;
	wire full;
	wire [11:0]q;
	wire [7:0]usedw;
	wire [7:0]tx_data;
	wire Set_Done;
	wire [7:0]data_byte;
	wire Rx_Done;
	wire [7:0]adc_din;
	wire [7:0]acq_num;
	wire [7:0]switch;
	
	
	output dac_cs_n;
	output dac_din;
	output dac_sclk;
	
	reg [15:0]r_data;
	wire [15:0]dac_data;
	reg dac_start;
	wire dac_state;
	wire START;
	
	reg [2:0]adc_din_r;
	
	adc_driver adc_driver(
		.Clk(Clk),
		.Rst_n(Rst_n),
		.START(1'd1),
		.DIN_R_IN(adc_din_r),
		.DOUT(DOUT),
		
		.cs(cs),
		.sclk_o(sclk_o),
		.dac_data(adc_data),
		.DIN(DIN),
		.RE_DONE(RE_DONE),
		.ADC_STATE(ADC_STATE),
		.ADC_START(ADC_START)
	);
	
	ctrl ctrl(
		.Clk(Clk),
		.Rst_n(Rst_n),
		.wrreq(wrreq),
		.rdreq(rdreq),
		.Key_flag(Key_flag),
		.Key_state(Key_state),
		
		.send_en(send_en),
		.ADC_START(ADC_START),
		//.DIN_R_IN(DIN_R_IN),
		.RE_DONE(RE_DONE),
		.ADC_STATE(ADC_STATE),
		.full(full),
		.fifo_data(q),
		.TX_DONE(Tx_Done),
		.tx_data(tx_data),
		.empty(empty),
		.usedw(usedw),
		.START(START),
		.acq_num(acq_num),
		.switch(switch)
	);
	
	key_filter key_filter(
		.Clk(Clk),
		.Rst_n(Rst_n),
		.key_in(Key_in),
		.key_flag(Key_flag),
		.key_state(Key_state)
	);
	
	
	
	myfifo myfifo(
		.clock(Clk),
		.data(adc_data),  //存入的为adc采集的数据，12位
		.rdreq(rdreq),
		.sclr(sclr),
		.wrreq(wrreq),
		.almost_empty(almost_empty),
		.almost_full(almost_full),  
		.empty(empty),  //empty:空  当存有数据的时候empty为0
		.full(full),  //空的时候full为高电平
		.q(q),
		.usedw(usedw)
	);
	
	uart_byte_tx uart_byte_tx(
		.Clk(Clk),
		.Rst_n(Rst_n),
		.data_byte(tx_data),
		.send_en(send_en),
		.baud_set(3'd4),
		
		.Rs232_Tx(Rs232_Tx),
		.Tx_Done(Tx_Done),
		.uart_state(uart_state)
	);
	


	tlv5618 tlv5618(
		.Clk(Clk),
		.Rst_n(Rst_n),
		
		.DAC_DATA(dac_data),
		.Start(dac_start),
		.Set_Done(Set_Done),
		
		.DAC_CS_N(dac_cs_n),
		.DAC_DIN(dac_din),
		.DAC_SCLK(dac_sclk),
		.DAC_State(dac_state)
	);
	
	
	myisp myisp(
		.probe(),
		.source(dac_data)
	);
	
	uart_byte_rx uart_byte_rx(
		.Clk(Clk),
		.Rst_n(Rst_n),
		.baud_set(3'd4),
		.Rs232_Rx(Rs232_Rx),
		
		.data_byte(data_byte),
		.Rx_Done(Rx_Done)
	);
	
//	always@(posedge Clk or negedge Rst_n)
//	if(!Rst_n)
//		rx_cnt = 8'd0;
//	else if(Rx_Done)
//		rx_cnt <= rx_cnt + 1'b1;
//	else if(rx_cnt == 6'd0)
		
	
	acq_uart_cmd acq_uart_cmd(
		.Clk(Clk),
		.Rst_n(Rst_n),
		
		.rx_data(data_byte),
		.rx_done(Rx_Done),
		.adc_din(adc_din),
		.acq_num(acq_num),
		.switch(switch),
		.led(led)
	);
	
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		r_data <= 16'd0;
	else if(dac_state)
		r_data <= dac_data;
	
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		dac_start <= 1'd0;
	else if(r_data != dac_data)
		dac_start <= 1'd1;
	else 
		dac_start <= 1'd0;
	
	//将接收的采样管脚对应到adc_driver
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		adc_din_r <= 3'd0;
	else begin
		case(adc_din)
			0: adc_din_r <= 3'd0;
			1: adc_din_r <= 3'd1;
			2: adc_din_r <= 3'd2;
			3: adc_din_r <= 3'd3;
			4: adc_din_r <= 3'd4;
			5: adc_din_r <= 3'd5;
			6: adc_din_r <= 3'd6;
			7: adc_din_r <= 3'd7;
			
		endcase
	end
	
	
endmodule 
	